// This file was generated by [ts-rs](https://github.com/Aleph-Alpha/ts-rs). Do not edit this file manually.
import type { FontSchemeFont } from "./FontSchemeFont";

export interface FontGroup { latin: string, ea: string, cs: string, fonts: Array<FontSchemeFont>, }